@@ -57,6 +57,8 @@ static enum fcft_subpixel subpixel_mode = FCFT_SUBPIXEL_DEFAULT;
 
 static pixman_color_t fg = {0x0000, 0x0000, 0x0000, 0xffff};
 static pixman_color_t bg = {0xffff, 0xffff, 0xffff, 0xffff};
+static enum fcft_scaling_filter scaling_filter = FCFT_SCALING_FILTER_CUBIC;
+static enum fcft_emoji_presentation emoji_presentation = FCFT_EMOJI_PRESENTATION_DEFAULT;
 
 static int width;
 static int height;
@@ -122,7 +124,7 @@ render_glyphs(struct buffer *buf, int *x, const int *y, pixman_image_t *color,
         if (kern != NULL)
             *x += kern[i];
 
-        if (pixman_image_get_format(g->pix) == PIXMAN_a8r8g8b8) {
+        if (g->is_color_glyph) {
             pixman_image_composite32(
                 PIXMAN_OP_OVER, g->pix, NULL, buf->pix, 0, 0, 0, 0,
                 *x + g->x, *y + font->ascent - g->y, g->width, g->height);
@@ -395,9 +397,11 @@ usage(const char *name)
         "\n"
         "Options:\n"
         "  -t,--text=TEXT            text string to render\n"
-        "  -f,--font=FONTS           comma separated list of FontConfig formatted font specifications\n"
-        "  -b,--background=RRGGBBAA  background color (e.g. ff000077 for semi-transparent red)\n"
-        "  -c,--foreground=RRGGBBAA  foreground color (e.g. 00ff00ff for non-transparent green)\n"
+        "  -f,--font=FONTS               comma separated list of FontConfig formatted font specifications\n"
+        "  -b,--background=RRGGBBAA      background color (e.g. ff000077 for semi-transparent red)\n"
+        "  -c,--foreground=RRGGBBAA      foreground color (e.g. 00ff00ff for non-transparent green)\n"
+        "  -s,--scaling-filter=FILT      pixman scaling filter to use when scaling color glyphs\n"
+        "  -e,--emoji-presentation=PRES  default emoji presentation\n"
         "  -h,--help                 show usage\n",
         name);
 }
@@ -428,6 +432,8 @@ main(int argc, char *const *argv)
         {"font",       required_argument, NULL, 'f'},
         {"background", required_argument, NULL, 'b'},
         {"foreground", required_argument, NULL, 'c'},
+        {"scaling-filter", required_argument, NULL, 's'},
+        {"emoji-presentation", required_argument, NULL, 'e'},
         {NULL,         no_argument,       NULL, '\0'},
     };
 
@@ -435,7 +441,7 @@ main(int argc, char *const *argv)
     const char *font_list = "serif:size=24";
 
     while (true) {
-        int c = getopt_long(argc, argv, "+t:f:b:c:h", options, NULL);
+        int c = getopt_long(argc, argv, "+t:f:b:c:s:e:h", options, NULL);
         if (c < 0)
             break;
 
@@ -474,6 +480,56 @@ main(int argc, char *const *argv)
             break;
         }
 
+        case 's':
+            if (strcmp(optarg, "none") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_NONE;
+            else if (strcmp(optarg, "nearest") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_NEAREST;
+            else if (strcmp(optarg, "bilinear") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_BILINEAR;
+            else if (strcmp(optarg, "impulse") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_IMPULSE;
+            else if (strcmp(optarg, "box") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_BOX;
+            else if (strcmp(optarg, "linear") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_LINEAR;
+            else if (strcmp(optarg, "cubic") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_CUBIC;
+            else if (strcmp(optarg, "gaussian") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_GAUSSIAN;
+            else if (strcmp(optarg, "lanczos2") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_LANCZOS2;
+            else if (strcmp(optarg, "lanczos3") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_LANCZOS3;
+            else if (strcmp(optarg, "lanczos3-stretched") == 0)
+                scaling_filter = FCFT_SCALING_FILTER_LANCZOS3_STRETCHED;
+            else {
+                fprintf(
+                    stderr,
+                    "%s: invalid scaling-filter, not one of 'none', "
+                    "'nearest', 'bilinear', 'impulse', 'box', 'linear', "
+                    "'cubic', 'gaussian', 'lanczos2', 'lanczos3', or "
+                    "'lanczos3-stretched'\n", optarg);
+                return EXIT_FAILURE;
+            }
+            break;
+
+        case 'e':
+            if (strcmp(optarg, "default") == 0)
+                emoji_presentation = FCFT_EMOJI_PRESENTATION_DEFAULT;
+            else if (strcmp(optarg, "text") == 0)
+                emoji_presentation = FCFT_EMOJI_PRESENTATION_TEXT;
+            else if (strcmp(optarg, "emoji") == 0)
+                emoji_presentation = FCFT_EMOJI_PRESENTATION_EMOJI;
+            else {
+                fprintf(
+                    stderr,
+                    "%s: invalid emoji-presentation, not one of 'default', "
+                    "'text' or 'emoji'\n", optarg);
+                return EXIT_FAILURE;
+            }
+            break;
+
         case 'h':
             usage(prog_name);
             return EXIT_SUCCESS;
@@ -570,10 +626,16 @@ main(int argc, char *const *argv)
         tll_foreach(font_names, it)
             names[idx++] = it->item;
 
-        font = fcft_from_name(tll_length(font_names), names, NULL);
-        assert(font != NULL);
-        fcft_set_emoji_presentation(font, FCFT_EMOJI_PRESENTATION_DEFAULT);
+        struct fcft_font_options *options = fcft_font_options_create();
+        /* Optionally set custom options */
 
+        options->scaling_filter = scaling_filter;
+        options->emoji_presentation = emoji_presentation;
+
+        font = fcft_from_name2(tll_length(font_names), names, NULL, options);
+        assert(font != NULL);
+
+        fcft_font_options_destroy(options);
         tll_free(font_names);
         free(copy);
     }
