@@ -53,7 +53,10 @@
 static FT_Library ft_lib = NULL;
 static mtx_t ft_lock;
 static bool can_set_lcd_filter = false;
-static enum fcft_scaling_filter scaling_filter = FCFT_SCALING_FILTER_CUBIC;
+
+/* fcft_set_scaling_filter() is deprecated; -1 means use the per-font
+   scaling filter */
+static enum fcft_scaling_filter scaling_filter = -1;
 
 static const size_t glyph_cache_initial_size = 256;
 #if defined(FCFT_HAVE_HARFBUZZ)
@@ -152,13 +155,20 @@ struct font_priv {
 #endif
 
     tll(struct fallback) fallbacks;
-    enum fcft_emoji_presentation emoji_presentation;
+
+    struct fcft_font_options options;
     size_t ref_counter;
 };
 
 /* Global font cache */
 struct fcft_font_cache_entry {
     uint64_t hash;
+
+    char **names;
+    char *attributes;
+    size_t name_count;
+    struct fcft_font_options options;
+
     struct font_priv *font;
 
     int waiters;
@@ -386,8 +396,14 @@ fcft_set_scaling_filter(enum fcft_scaling_filter filter)
     case FCFT_SCALING_FILTER_NONE:
     case FCFT_SCALING_FILTER_NEAREST:
     case FCFT_SCALING_FILTER_BILINEAR:
+    case FCFT_SCALING_FILTER_IMPULSE:
+    case FCFT_SCALING_FILTER_BOX:
+    case FCFT_SCALING_FILTER_LINEAR:
     case FCFT_SCALING_FILTER_CUBIC:
+    case FCFT_SCALING_FILTER_GAUSSIAN:
+    case FCFT_SCALING_FILTER_LANCZOS2:
     case FCFT_SCALING_FILTER_LANCZOS3:
+    case FCFT_SCALING_FILTER_LANCZOS3_STRETCHED:
         scaling_filter = filter;
         return true;
     }
@@ -899,7 +915,8 @@ sdbm_hash(const char *s)
 }
 
 static uint64_t
-font_hash(size_t count, const char *names[static count], const char *attributes)
+font_hash(size_t count, const char *names[static count], const char *attributes,
+          const struct fcft_font_options *options)
 {
     uint64_t hash = 0;
     for (size_t i = 0; i < count; i++)
@@ -908,12 +925,51 @@ font_hash(size_t count, const char *names[static count], const char *attributes)
     if (attributes != NULL)
         hash ^= sdbm_hash(attributes);
 
+    _Static_assert(sizeof(*options) % 8 == 0, "struct fcft_font_options not a multiple of 8 bytes");
+
+    for (size_t i = 0; i < sizeof(*options) / 8; i++) {
+        const uint64_t v = ((const uint16_t *)options)[i];
+        hash ^= v;
+    }
+
     return hash;
 }
 
+static const struct fcft_font_options default_font_options = {
+    .scaling_filter = FCFT_SCALING_FILTER_CUBIC,
+    .emoji_presentation = FCFT_EMOJI_PRESENTATION_DEFAULT,
+    .color_glyphs = {
+        .srgb_decode = false,
+        .format = PIXMAN_a8r8g8b8,
+    },
+};
+
+FCFT_EXPORT struct fcft_font_options *
+fcft_font_options_create(void)
+{
+    struct fcft_font_options *ret = calloc(1, sizeof(*ret));
+    *ret = default_font_options;
+    return ret;
+}
+
+FCFT_EXPORT void
+fcft_font_options_destroy(struct fcft_font_options *options)
+{
+    free(options);
+}
+
 FCFT_EXPORT struct fcft_font *
 fcft_from_name(size_t count, const char *names[static count],
                const char *attributes)
+{
+    return fcft_from_name2(count, names, attributes, NULL);
+}
+
+
+FCFT_EXPORT struct fcft_font *
+fcft_from_name2(
+    size_t count, const char *names[static count], const char *attributes,
+    const struct fcft_font_options *options)
 {
     if (ft_lib == NULL) {
         LOG_ERR("fcft_init() not called");
@@ -925,7 +981,19 @@ fcft_from_name(size_t count, const char *names[static count],
     if (count == 0)
         return NULL;
 
-    uint64_t hash = font_hash(count, names, attributes);
+    if (options == NULL)
+        options = &default_font_options;
+
+#if 0
+    if (options->color_glyphs.format != PIXMAN_a8r8g8b8 &&
+        options->color_glyphs.format != PIXMAN_a2r10g10b10 &&
+        options->color_glyphs.format != PIXMAN_a2b10g10r10)
+    {
+        /* Unsupported format */
+        return NULL;
+    }
+#endif
+    uint64_t hash = font_hash(count, names, attributes, options);
     struct fcft_font_cache_entry *cache_entry = NULL;
 
     mtx_lock(&font_cache_lock);
@@ -933,6 +1001,29 @@ fcft_from_name(size_t count, const char *names[static count],
         if (it->item.hash != hash)
             continue;
 
+        if (memcmp(&it->item.options, options, sizeof(*options)) != 0)
+            continue;
+
+        if (it->item.name_count != count)
+            continue;
+
+        if ((attributes == NULL && it->item.attributes != NULL) ||
+            (attributes != NULL && it->item.attributes == NULL))
+        {
+            continue;
+        }
+
+        if (attributes != NULL &&
+            strcmp(it->item.attributes, attributes) != 0)
+        {
+            continue;
+        }
+
+        for (size_t i = 0; i < count; i++) {
+            if (strcmp(it->item.names[i], names[i]) != 0)
+                continue;
+        }
+
         struct fcft_font_cache_entry *e = &it->item;
 
         if (e->font != (void *)(uintptr_t)-1) {
@@ -970,6 +1061,14 @@ fcft_from_name(size_t count, const char *names[static count],
 
     cache_entry = &tll_back(font_cache);
 
+    cache_entry->name_count = count;
+    cache_entry->names = malloc(count * sizeof(char *));
+    cache_entry->attributes = attributes != NULL ? strdup(attributes) : NULL;
+    memcpy(&cache_entry->options, options, sizeof(*options));
+
+    for (size_t i = 0; i < count; i++)
+        cache_entry->names[i] = strdup(names[i]);
+
     if (cnd_init(&cache_entry->cond) != thrd_success) {
         LOG_ERR("%s: failed to instantiate font cache condition variable", names[0]);
         tll_pop_back(font_cache);
@@ -1032,14 +1131,16 @@ fcft_from_name(size_t count, const char *names[static count],
             bool grapheme_cache_lock_failed = true;
             bool pattern_failed = true;
 
-            mtx_t lock;
-            if (mtx_init(&lock, mtx_plain) != thrd_success)
+            font = calloc(1, sizeof(*font));
+            if (font == NULL)
+                goto fc_destroy;
+
+            if (mtx_init(&font->lock, mtx_plain) != thrd_success)
                 LOG_WARN("%s: failed to instantiate mutex", name);
             else
                 lock_failed = false;
 
-            pthread_rwlock_t glyph_cache_lock;
-            if (pthread_rwlock_init(&glyph_cache_lock, NULL) != 0)
+            if (pthread_rwlock_init(&font->glyph_cache_lock, NULL) != 0)
                 LOG_WARN("%s: failed to instantiate glyph cache rwlock", name);
             else
                 glyph_cache_lock_failed = false;
@@ -1052,13 +1153,11 @@ fcft_from_name(size_t count, const char *names[static count],
             } else
                 pattern_failed = false;
 
-            font = calloc(1, sizeof(*font));
             struct glyph_priv **glyph_cache_table = calloc(
                 glyph_cache_initial_size, sizeof(glyph_cache_table[0]));
 
 #if defined(FCFT_HAVE_HARFBUZZ)
-            pthread_rwlock_t grapheme_cache_lock;
-            if (pthread_rwlock_init(&grapheme_cache_lock, NULL) != 0)
+            if (pthread_rwlock_init(&font->grapheme_cache_lock, NULL) != 0)
                 LOG_WARN("%s: failed to instantiate grapheme cache rwlock", name);
             else
                 grapheme_cache_lock_failed = false;
@@ -1073,25 +1172,27 @@ fcft_from_name(size_t count, const char *names[static count],
             /* Handle failure(s) */
             if (lock_failed || glyph_cache_lock_failed ||
                 grapheme_cache_lock_failed || pattern_failed ||
-                font == NULL || glyph_cache_table == NULL
+                glyph_cache_table == NULL
 #if defined(FCFT_HAVE_HARFBUZZ)
                 || grapheme_cache_table == NULL
 #endif
                 )
             {
                 if (!lock_failed)
-                    mtx_destroy(&lock);
+                    mtx_destroy(&font->lock);
                 if (!glyph_cache_lock_failed)
-                    pthread_rwlock_destroy(&glyph_cache_lock);
+                    pthread_rwlock_destroy(&font->glyph_cache_lock);
 #if defined(FCFT_HAVE_HARFBUZZ)
                 if (!grapheme_cache_lock_failed)
-                    pthread_rwlock_destroy(&grapheme_cache_lock);
+                    pthread_rwlock_destroy(&font->grapheme_cache_lock);
 #endif
                 if (!pattern_failed)
                     free(primary);
                 free(font);
+                font = NULL;
                 free(glyph_cache_table);
                 free(grapheme_cache_table);
+fc_destroy:
                 if (langset != NULL)
                     FcLangSetDestroy(langset);
                 FcCharSetDestroy(charset);
@@ -1102,16 +1203,13 @@ fcft_from_name(size_t count, const char *names[static count],
             }
 
             font->ref_counter = 1;
-            font->lock = lock;
-            font->glyph_cache_lock = glyph_cache_lock;
             font->glyph_cache.size = glyph_cache_initial_size;
             font->glyph_cache.count = 0;
             font->glyph_cache.table = glyph_cache_table;
-            font->emoji_presentation = FCFT_EMOJI_PRESENTATION_DEFAULT;
+            font->options = *options;
             font->public = primary->metrics;
 
 #if defined(FCFT_HAVE_HARFBUZZ)
-            font->grapheme_cache_lock = grapheme_cache_lock;
             font->grapheme_cache.size = grapheme_cache_initial_size;
             font->grapheme_cache.count = 0;
             font->grapheme_cache.table = grapheme_cache_table;
@@ -1206,7 +1304,9 @@ fcft_clone(const struct fcft_font *_font)
 
 static bool
 glyph_for_index(const struct instance *inst, uint32_t index,
-                enum fcft_subpixel subpixel, struct glyph_priv *glyph)
+                enum fcft_subpixel subpixel,
+                const struct fcft_font_options *options,
+                struct glyph_priv *glyph)
 {
     glyph->valid = false;
     glyph->subpixel = subpixel;
@@ -1336,7 +1436,9 @@ glyph_for_index(const struct instance *inst, uint32_t index,
         break;
 
     case FT_PIXEL_MODE_BGRA:
-        pix_format = PIXMAN_a8r8g8b8;
+        pix_format = options->color_glyphs.srgb_decode
+            ? PIXMAN_a8r8g8b8_sRGB
+            : PIXMAN_a8r8g8b8;
         width = bitmap->width;
         rows = bitmap->rows;
         break;
@@ -1396,7 +1498,7 @@ glyph_for_index(const struct instance *inst, uint32_t index,
         }
         break;
 
-    case FT_PIXEL_MODE_BGRA: /* PIXMAN_a8r8g8b8 */
+    case FT_PIXEL_MODE_BGRA: /* PIXMAN_a8r8g8b8 by default */
         /*
          * FreeType: blue comes *first* in memory
          * Pixman: LE: blue comes *first* in memory
@@ -1418,6 +1520,7 @@ glyph_for_index(const struct instance *inst, uint32_t index,
                 *p = (uint32_t)_a << 24 | _r << 16 | _g << 8 | _b;
             }
         }
+
         break;
 
     case FT_PIXEL_MODE_LCD: /* PIXMAN_x8r8g8b8 */
@@ -1471,6 +1574,51 @@ glyph_for_index(const struct instance *inst, uint32_t index,
              pix_format, width, rows, (uint32_t *)data, stride)) == NULL)
         goto err;
 
+    if (bitmap->pixel_mode == FT_PIXEL_MODE_BGRA &&
+        options->color_glyphs.format != pix_format)
+    {
+        /*
+         * Backing buffer is currently in sRGB, a8r8g8b8. Need to
+         * decode and/or convert to requested output format.
+         */
+        assert(pix_format == PIXMAN_a8r8g8b8 ||
+               pix_format == PIXMAN_a8r8g8b8_sRGB);
+
+        /* Old pixman image + backing buffer will be replaced */
+        pixman_image_t *a8r8g8b8 = pix;
+        uint8_t *a8_data = data;
+
+        /* New backing buffer */
+        pix_format = options->color_glyphs.format;
+        stride = stride_for_format_and_width(pix_format, width);
+        data = malloc(rows * stride);
+
+        if (data == NULL) {
+            data = a8_data;
+            pix = a8r8g8b8;
+            goto err;
+        }
+
+        /* User requested output format */
+        pix = pixman_image_create_bits_no_clear(
+            pix_format, width, rows, (uint32_t *)data, stride);
+
+        if (pix == NULL) {
+            free(data);
+            data = a8_data;
+            pix = a8r8g8b8;
+            goto err;
+        }
+
+        /* "copy" a8r8g8b8 source to user requested output
+           format. This also decodes from sRGB, if requested */
+        pixman_image_composite32(
+            PIXMAN_OP_SRC, a8r8g8b8, NULL, pix, 0, 0, 0, 0, 0, 0, width, rows);
+
+        pixman_image_unref(a8r8g8b8);
+        free(a8_data);
+    }
+
     pixman_image_set_component_alpha(
         pix,
         bitmap->pixel_mode == FT_PIXEL_MODE_LCD ||
@@ -1494,8 +1642,10 @@ glyph_for_index(const struct instance *inst, uint32_t index,
         pixman_image_set_transform(pix, &_scale);
 
         const enum fcft_scaling_filter filter_to_use = inst->is_color
-            ? scaling_filter
-            : FCFT_SCALING_FILTER_BILINEAR;
+            ? (scaling_filter != (enum fcft_scaling_filter)-1
+                ? scaling_filter
+                : options->scaling_filter)
+            : FCFT_SCALING_FILTER_NONE;
 
         switch (filter_to_use) {
         case FCFT_SCALING_FILTER_NONE:
@@ -1509,16 +1659,33 @@ glyph_for_index(const struct instance *inst, uint32_t index,
             pixman_image_set_filter(pix, PIXMAN_FILTER_BILINEAR, NULL, 0);
             break;
 
+        case FCFT_SCALING_FILTER_IMPULSE:
+        case FCFT_SCALING_FILTER_BOX:
+        case FCFT_SCALING_FILTER_LINEAR:
         case FCFT_SCALING_FILTER_CUBIC:
-        case FCFT_SCALING_FILTER_LANCZOS3: {
+        case FCFT_SCALING_FILTER_GAUSSIAN:
+        case FCFT_SCALING_FILTER_LANCZOS2:
+        case FCFT_SCALING_FILTER_LANCZOS3:
+        case FCFT_SCALING_FILTER_LANCZOS3_STRETCHED: {
             /*
              * TODO:
              *   - find out how the subsample_bit_{x,y} parameters should be set
              */
             int param_count = 0;
-            pixman_kernel_t kernel = filter_to_use == FCFT_SCALING_FILTER_CUBIC
-                ? PIXMAN_KERNEL_CUBIC
-                : PIXMAN_KERNEL_LANCZOS3;
+            pixman_kernel_t kernel;
+
+            switch (filter_to_use) {
+            case FCFT_SCALING_FILTER_IMPULSE: kernel = PIXMAN_KERNEL_IMPULSE; break;
+            case FCFT_SCALING_FILTER_BOX: kernel = PIXMAN_KERNEL_BOX; break;
+            case FCFT_SCALING_FILTER_LINEAR: kernel = PIXMAN_KERNEL_LINEAR; break;
+            case FCFT_SCALING_FILTER_CUBIC: kernel = PIXMAN_KERNEL_CUBIC; break;
+            case FCFT_SCALING_FILTER_GAUSSIAN: kernel = PIXMAN_KERNEL_GAUSSIAN; break;
+            case FCFT_SCALING_FILTER_LANCZOS2: kernel = PIXMAN_KERNEL_LANCZOS2; break;
+            case FCFT_SCALING_FILTER_LANCZOS3: kernel = PIXMAN_KERNEL_LANCZOS3; break;
+            case FCFT_SCALING_FILTER_LANCZOS3_STRETCHED:  kernel = PIXMAN_KERNEL_LANCZOS3_STRETCHED; break;
+
+            default: assert(false); kernel = PIXMAN_KERNEL_CUBIC; break;
+            }
 
             pixman_fixed_t *params = pixman_filter_create_separable_convolution(
                 &param_count,
@@ -1541,7 +1708,7 @@ glyph_for_index(const struct instance *inst, uint32_t index,
         int scaled_rows = rows / (1. / inst->pixel_size_fixup);
         int scaled_stride = stride_for_format_and_width(pix_format, scaled_width);
 
-        if (pix_format == PIXMAN_a8r8g8b8) {
+        if (bitmap->pixel_mode == FT_PIXEL_MODE_BGRA) {
             uint8_t *scaled_data = malloc(scaled_rows * scaled_stride);
             if (scaled_data == NULL)
                 goto err;
@@ -1588,6 +1755,7 @@ glyph_for_index(const struct instance *inst, uint32_t index,
             },
             .width = width,
             .height = rows,
+            .is_color_glyph = bitmap->pixel_mode == FT_PIXEL_MODE_BGRA,
         },
         .subpixel = subpixel,
         .valid = true,
@@ -1605,7 +1773,9 @@ err:
 
 static bool
 glyph_for_codepoint(const struct instance *inst, uint32_t cp,
-                    enum fcft_subpixel subpixel, struct glyph_priv *glyph)
+                    enum fcft_subpixel subpixel,
+                    const struct fcft_font_options *options,
+                    struct glyph_priv *glyph)
 {
     FT_UInt idx = -1;
 
@@ -1634,7 +1804,7 @@ glyph_for_codepoint(const struct instance *inst, uint32_t cp,
     if (idx == (FT_UInt)-1)
         idx = FT_Get_Char_Index(inst->face, cp);
 
-    bool ret = glyph_for_index(inst, idx, subpixel, glyph);
+    bool ret = glyph_for_index(inst, idx, subpixel, options, glyph);
     glyph->public.cp = cp;
     glyph->public.cols = cpwidth(cp);
     return ret;
@@ -1815,8 +1985,8 @@ fcft_rasterize_char_utf32(struct fcft_font *_font, uint32_t cp,
     bool force_emoji_presentation = false;
     bool enforce_presentation_style = emoji != NULL;
 
-    if (emoji != NULL) {
-        switch (font->emoji_presentation) {
+    if (emoji != NULL && !emoji->component) {
+        switch (font->options.emoji_presentation) {
         case FCFT_EMOJI_PRESENTATION_TEXT:
             force_text_presentation = true;
             force_emoji_presentation = false;
@@ -1879,7 +2049,7 @@ search_fonts:
         }
 
         assert(it->item.font != NULL);
-        got_glyph = glyph_for_codepoint(it->item.font, cp, subpixel, glyph);
+        got_glyph = glyph_for_codepoint(it->item.font, cp, subpixel, &font->options, glyph);
         no_one = false;
         break;
     }
@@ -1897,7 +2067,7 @@ search_fonts:
         struct instance *inst = tll_front(font->fallbacks).font;
 
         assert(inst != NULL);
-        got_glyph = glyph_for_codepoint(inst, cp, subpixel, glyph);
+        got_glyph = glyph_for_codepoint(inst, cp, subpixel, &font->options, glyph);
     }
 
     assert(*entry == NULL);
@@ -2031,7 +2201,7 @@ font_for_grapheme(struct font_priv *font,
                                      cluster[i] < emoji->cp + emoji->count));
 
             if (enforce_presentation_style &&
-                emoji != NULL &&
+                emoji != NULL && !emoji->component &&
                 (i + 1 >= len || (cluster[i + 1] != 0xfe0e &&
                                   cluster[i + 1] != 0xfe0f))) {
                 /*
@@ -2042,7 +2212,7 @@ font_for_grapheme(struct font_priv *font,
                 bool force_text_presentation = false;
                 bool force_emoji_presentation = false;
 
-                switch (font->emoji_presentation) {
+                switch (font->options.emoji_presentation) {
                 case FCFT_EMOJI_PRESENTATION_TEXT:
                     force_text_presentation = true;
                     force_emoji_presentation = false;
@@ -2242,7 +2412,7 @@ fcft_rasterize_grapheme_utf32(struct fcft_font *_font,
 
         struct glyph_priv *glyph = malloc(sizeof(*glyph));
         if (glyph == NULL ||
-            !glyph_for_index(inst, info[i].codepoint, subpixel, glyph))
+            !glyph_for_index(inst, info[i].codepoint, subpixel, &font->options, glyph))
         {
             assert(glyph == NULL || !glyph->valid);
             free(glyph);
@@ -2335,7 +2505,8 @@ static bool
 rasterize_partial_run(struct text_run *run, const struct instance *inst,
                       const uint32_t *text, size_t len,
                       size_t run_start, size_t run_len,
-                      enum fcft_subpixel subpixel)
+                      enum fcft_subpixel subpixel,
+                      const struct fcft_font_options *options)
 {
     hb_buffer_add_utf32(inst->hb_buf, text, len, run_start, run_len);
     hb_buffer_guess_segment_properties(inst->hb_buf);
@@ -2369,7 +2540,7 @@ rasterize_partial_run(struct text_run *run, const struct instance *inst,
         if (glyph == NULL)
             return false;
 
-        if (!glyph_for_index(inst, info->codepoint, subpixel, glyph)) {
+        if (!glyph_for_index(inst, info->codepoint, subpixel, options, glyph)) {
             free(glyph);
             continue;
         }
@@ -2553,7 +2724,7 @@ fcft_rasterize_text_run_utf32(
 
         bool ret = rasterize_partial_run(
             &run, prun->inst, (const uint32_t *)text, len,
-            prun->start, prun->len, subpixel);
+            prun->start, prun->len, subpixel, &font->options);
 
         hb_buffer_clear_contents(prun->inst->hb_buf);
         if (!ret)
@@ -2655,6 +2826,12 @@ fcft_destroy(struct fcft_font *_font)
             mtx_unlock(&font->lock);
 
             cnd_destroy(&it->item.cond);
+
+            for (size_t i = 0; i < it->item.name_count; i++)
+                free(it->item.names[i]);
+            free(it->item.names);
+            free(it->item.attributes);
+
             tll_remove(font_cache, it);
             break;
         }
@@ -2848,7 +3025,7 @@ fcft_set_emoji_presentation(struct fcft_font *_font,
                             enum fcft_emoji_presentation presentation)
 {
     struct font_priv *font = (struct font_priv *)_font;
-    font->emoji_presentation = presentation;
+    font->options.emoji_presentation = presentation;
 }
 
 static int
