@@ -3,4 +3,4 @@
 #include <math.h>
 #define NANOSVG_ALL_COLOR_KEYWORDS
 #define NANOSVG_IMPLEMENTATION
-#include <nanosvg.h>
+#include <3rd-party/nanosvg/src/nanosvg.h>
